/****************************************************************
 * Skyler Schneider ss868                                       *
 * Jack Ellsworth   jfe5                                        *
 * ECE 4760 Final Project                                       *
 * USB Wireless Tilt Mouse                                      *
 ****************************************************************/

// Base station consists of USB and wireless RX.
// This unit receives data from RX MCU and uses it for USB or a game.

/****************************************************************
 * V-USB HID Mouse Example Information                          *
 * Code was used for this project.                              *
 ****************************************************************/

/* Name: main.c
 * Project: hid-mouse, a very simple HID example
 * Author: Christian Starkjohann
 * Creation Date: 2008-04-07
 * Tabsize: 4
 * Copyright: (c) 2008 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt), GNU GPL v3 or proprietary (CommercialLicense.txt)
 * This Revision: $Id: main.c 692 2008-11-07 15:07:40Z cs $
 */

/*
This example should run on most AVRs with only little changes. No special
hardware resources except INT0 are used. You may have to change usbconfig.h for
different I/O pins for USB. Please note that USB D+ must be the INT0 pin, or
at least be connected to INT0 as well.

We use VID/PID 0x046D/0xC00E which is taken from a Logitech mouse. Don't
publish any hardware using these IDs! This is for demonstration only!
*/

/****************************************************************
 * NTSC Video Example Information                               *
 * Code was used for this project.                              *
 ****************************************************************/

// Black and white NTSC video generation with fixed point animation
// Mega644 version by Shane Pryor 
// Mods by brl4@cornell.edu
// Thanks to Jonathan Diamond for debugging help.

/****************************************************************
 * Hardware Connections                                         *
 ****************************************************************/

/*
 * PORTB
 *  B.0 - Data from RX MCU
 *  B.1 - Data from RX MCU
 *  B.2 - Data from RX MCU
 *  B.3 - Data from RX MCU
 *  B.5 - Addr from RX MCU
 *  B.6 - Addr from RX MCU
 *  B.7 - Addr from RX MCU
 */

/*
 * PORTC
 *  C.0 - Toggle for game/USB
 *  C.5 - Video: 1000 ohm to 75 ohm resistor
 *  C.6 - Sync: 300 ohm to 75 ohm resistor
 */

/*
 * PORTD
 *  D.0 - Wireless receiver
 *  D.2 - USB D+
 *  D.4 - USB D-
 *  D.5 - ACK to RX MCU
 *  D.6 - ACK to RX MCU
 *  D.7 - ACK to RX MCU
 */

/****************************************************************
 * Compiler Directives                                          *
 ****************************************************************/

#include <inttypes.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <util/delay.h>
#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>

#include "drawRoutines.h"

#include "usbdrv.h"
#include "usbinterface.h"  // USB report descriptor

// Cycles = 63.625 * 16
// Note: NTSC is 63.55, but this line duration makes each frame
// exactly 1/60 sec, which is nice for keeping a realtime clock
#define lineTime  1018
#define ScreenTop 30
#define ScreenBot 230

#define G_STATE_CASCADE 1
#define G_STATE_LOSE    50
#define G_STATE_WIN     100
#define G_STATE_END     255

/****************************************************************
 * Global Declarations                                          *
 ****************************************************************/

// Structure used for communicating between RX and USB units.
typedef struct {
    char buttonMask;
    char dxL;
    char dxH;
    char dyL;
    char dyH;
    char dWheelL;
    char dWheelH;
} txrx_usb_buf_t;
txrx_usb_buf_t rx_usb_buf;

// Structure used for communicating between USB and PC (see usbinterface.h).
report_t reportBuffer;
uint8_t  idleRate; // repeat rate for keyboards, never used for mice

// Sync variables for video generation
volatile char syncON, syncOFF;

// Current line number in the current frame
volatile int LineCount;
volatile int HalfVisibleLineCount;

// Game variables
int16_t  g_xCursor;
int16_t  g_yCursor;
uint8_t  g_leftButton;
uint8_t  g_leftButton_prev;
uint8_t  g_middleButton;
uint8_t  g_rightButton;
uint8_t  g_rightButton_prev;
int8_t   g_vx;
int8_t   g_vy;
int16_t  g_counting;
int8_t   g_mouseOver_row;
int8_t   g_mouseOver_col;
int8_t   g_mouseOver;
int8_t   g_mouseOver_row_prev;
int8_t   g_mouseOver_col_prev;
int8_t   g_mouseOver_prev;
int8_t   g_clicked;
uint8_t  g_gameField[81];
uint8_t  g_inCascade[81];
int8_t   g_cascadeQueue[81];
uint8_t  g_CQsetterIndex;
uint8_t  g_CQserviceIndex;
uint8_t  g_timeCntr;
uint16_t g_timeSec;
uint8_t  g_state;
uint8_t  g_minesRemaining;
uint8_t  g_tilesRemaining;

// Variable for choosing between modes of operation.
uint8_t operationMode;

void init(void);   // initialization method
void reinit(void); // re-initialization method

/****************************************************************
 * Interrupt Service Routines                                   *
 ****************************************************************/

// This is the sync generator and raster generator. It MUST be entered from
// sleep mode to get accurate timing of the sync pulses.
ISR (TIMER1_COMPA_vect) {
    
    // Start the Horizontal sync pulse
    PORTC = syncON;
    
    // Update the current scanline number
    LineCount++;
    
    // Begin inverted (Vertical) synch after line 247
    if (LineCount==248) {
        syncON = 0b01000000;
        syncOFF = 0;
    }
    
    // Back to regular sync after line 250
    if (LineCount==251) {
        syncON = 0;
        syncOFF = 0b01000000;
    }
    
    // Start new frame after line 262
    if (LineCount==263) LineCount = 1;
    
    // Adjust to make 5 us pulses
    _delay_us(2);
    
    // End sync pulse
    PORTC = syncOFF;
    
    if (LineCount < ScreenBot && LineCount >= ScreenTop) {
        // Compute offset into screen array
        // modified for 9 bytes per screen, not 18
        HalfVisibleLineCount = (LineCount - ScreenTop) >> 1;
        screenindex = (int *)(screen + (HalfVisibleLineCount << 3) + HalfVisibleLineCount);
        
        // Center image on screen
        _delay_us(9);
        
        // Blast the data to the screen
        byteblast();
    }
}

/****************************************************************
 * Helper Functions                                             *
 ****************************************************************/

// USB device function setup.
usbMsgLen_t usbFunctionSetup(uchar data[8]) {
    usbRequest_t *rq = (void *)data;
    // The following requests are never used. But since they are required by
    // the specification, we implement them in this example.
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS) { // class request type
        if(rq->bRequest == USBRQ_HID_GET_REPORT) { // wValue: ReportType (highbyte), ReportID (lowbyte)
            // we only have one report type, so don't look at wValue
            usbMsgPtr = (void *)&reportBuffer;
            return sizeof(reportBuffer);
        }
        else if(rq->bRequest == USBRQ_HID_GET_IDLE) {
            usbMsgPtr = &idleRate;
            return 1;
        } else if(rq->bRequest == USBRQ_HID_SET_IDLE) {
            idleRate = rq->wValue.bytes[1];
        }
    }
    else {
        // no vendor specific requests implemented
    }
    return 0; // default for not implemented requests: return no data back to host
}

// Decodes an 8-bit signal from PINB into a 4-bit addr and 4-bit data.
void decodePINB(char PINB_read) {
    switch(PINB_read & 0b11100000) {
        case 0b00100000:
            rx_usb_buf.buttonMask = PINB_read & 0x0F;
            break;
        case 0b01000000:
            rx_usb_buf.dxL = PINB_read & 0x0F;
            break;
        case 0b01100000:
            rx_usb_buf.dxH = PINB_read & 0x0F;
            break;
        case 0b10000000:
            rx_usb_buf.dyL = PINB_read & 0x0F;
            break;
        case 0b10100000:
            rx_usb_buf.dyH = PINB_read & 0x0F;
            break;
        case 0b11000000:
            rx_usb_buf.dWheelL = PINB_read & 0x0F;
            break;
        case 0b11100000:
            rx_usb_buf.dWheelH = PINB_read & 0x0F;
            break;
        default:
            break;
    }
}

// Given a row and column, returns whether the specified tile is a buried mine.
uint8_t isBuriedMine(uint8_t row, uint8_t col) {
    uint8_t tileNum = (row*9) + col;
    return (g_gameField[tileNum] == 0x0F);
}

// Calculates and assigns number of neighboring mines of tile tileNum.
void calcNeighboringMines(uint8_t tileNum) {
    uint8_t row = tileNum / 9;
    uint8_t col = tileNum % 9;
    uint8_t numMines = 0;
    if(isBuriedMine(row,col)) return;
    if(row > 0) {
        if(col > 0) numMines += isBuriedMine(row-1, col-1);
        if(col < 8) numMines += isBuriedMine(row-1, col+1);
                    numMines += isBuriedMine(row-1, col);
    }
    if(row < 8) {
        if(col > 0) numMines += isBuriedMine(row+1, col-1);
        if(col < 8) numMines += isBuriedMine(row+1, col+1);
                    numMines += isBuriedMine(row+1, col);
    }
    if(col > 0)     numMines += isBuriedMine(row, col-1);
    if(col < 8)     numMines += isBuriedMine(row, col+1);
    g_gameField[tileNum] = numMines;
}

// Initializes the minefield for beginning of game.
void setupField(void) {
    // Reset all tiles to empty, unclicked
    uint8_t i;
    for(i = 0; i < 81; i++) {
        g_gameField[i] = 0x00;
    }
    uint8_t minesLeft = 10;
    while(minesLeft) {
        i = rand() % 81;
        if(g_gameField[i] == 0x00) {
            // bury mine
            g_gameField[i] = 0x0F;
            minesLeft--;
        }
    }
    for(i = 0; i < 81; i++) {
        calcNeighboringMines(i);
    }
}

// Recursively identifies which tiles on the game field are in a cascade.
void cascadeHelper(int8_t row, int8_t col, uint8_t first) {
    int8_t i = (row*9)+col;
    if(g_inCascade[i]) return;
    if(first || (((g_gameField[i] & 0xF0) == 0x00) && (g_gameField[i] != 0x0F))) {
        g_inCascade[i] = 1;
        if(first || g_gameField[i] == 0x00) {
            if(row > 0) cascadeHelper(row-1, col, 0);
            if(row < 8) cascadeHelper(row+1, col, 0);
            if(col > 0) cascadeHelper(row, col-1, 0);
            if(col < 8) cascadeHelper(row, col+1, 0);
        }
    }
}

/****************************************************************
 * Main Routines                                                *
 ****************************************************************/

void main_game(void) {
    uint8_t i = 0;
    uint8_t j = 0;
    uint8_t k = 0;
    int16_t xCursor_temp = 0;
    int16_t yCursor_temp = 0;
    char PINB_read = 0;
    char PINB_read_0 = 0;
    char PINB_read_1 = 0;
    char PINB_read_2 = 0;
    char g_delay = 0;

    // The following loop executes once/video line during lines
    // 1-230, then does all of the frame-end processing
    while(1) {
        // Stall here until next line starts.
        // Use sleep to make entry into sync ISR uniform time
        sleep_cpu();
        
        // The following code executes during the vertical blanking code here can
        // be as long as a total of 60 lines x 63.5 uSec/line x 8 cycles/uSec
        if (LineCount == 231) {
            
            // undraw previous cursor
            undrawCursor();
            
            // Detect transition from unpressed to pressed
            g_leftButton_prev  = g_leftButton;
            g_leftButton       = rx_usb_buf.buttonMask & 0b00000001;
            g_middleButton     = rx_usb_buf.buttonMask & 0b00000100;
            g_rightButton_prev = g_rightButton;
            g_rightButton      = rx_usb_buf.buttonMask & 0b00000010;
            if(g_vx || g_vy) {
                g_vx = 0;
                g_vy = 0;
                g_delay = 2;
            }
            else {
                if(g_delay > 0) g_delay--;
                else {
                    g_vx = ((rx_usb_buf.dxH << 4) | rx_usb_buf.dxL);
                    g_vy = ((rx_usb_buf.dyH << 4) | rx_usb_buf.dyL);
                    g_vx = g_vx >> 2;
                    g_vy = g_vy >> 2;
                }
            }
            
            if(g_middleButton && g_rightButton) {
                break;
            }
            
            // update cursor position
            xCursor_temp = g_xCursor;
            yCursor_temp = g_yCursor;
            g_xCursor += g_vx;
            g_yCursor += g_vy;
            if(g_xCursor < 0) {
                g_xCursor -= g_vx;
                g_xCursor -= xCursor_temp;
            }
            if(g_yCursor < 28) {
                g_yCursor -= g_vy;
                g_yCursor -= yCursor_temp;
                g_yCursor += 28;
            }
            if(g_xCursor > 70) {
                g_xCursor -= g_vx;
                g_xCursor -= xCursor_temp;
                g_xCursor += 70;
            }
            if(g_yCursor > 97) {
                g_yCursor -= g_vy;
                g_yCursor -= yCursor_temp;
                g_yCursor += 97;
            }
            
            if(g_state == 0) {
                // update and draw time
                g_timeCntr--;
                if(!g_timeCntr) {
                    g_timeCntr = 60;
                    g_timeSec++;
                    if(g_timeSec > 999) g_timeSec = 999;
                    drawDec8(57, 12, g_timeSec);
                }
                
                // undraw/redraw previous highlighted thing if it exists and is different
                g_mouseOver_prev     = g_mouseOver;
                g_mouseOver_row_prev = g_mouseOver_row;
                g_mouseOver_col_prev = g_mouseOver_col;
                g_mouseOver_row = (g_yCursor-28) / 8;
                g_mouseOver_col = g_xCursor / 8;
                g_mouseOver = (g_mouseOver_row*9) + g_mouseOver_col;
                
                if(g_leftButton_prev && ((g_gameField[g_mouseOver_prev] & 0xF0) == 0x00)) {
                    drawSprite(g_mouseOver_col_prev*8, g_mouseOver_row_prev*8 + 28, 3);
                    drawSprite(g_mouseOver_col_prev*8, g_mouseOver_row_prev*8 + 28, 2);
                }
                if(g_leftButton && ((g_gameField[g_mouseOver] & 0xF0) == 0x00)) {
                    drawSprite(g_mouseOver_col*8, g_mouseOver_row*8 + 28, 2);
                    drawSprite(g_mouseOver_col*8, g_mouseOver_row*8 + 28, 3);
                }
                
                // undraw/redraw clicked thing if it exists
                if((!g_leftButton) && g_leftButton_prev && ((g_gameField[g_mouseOver] & 0xF0) == 0x00)) {
                    // left-clicked mouse
                    drawSprite(g_mouseOver_col*8, g_mouseOver_row*8 + 28, 2);
                    g_clicked = g_gameField[g_mouseOver];
                    if(g_clicked == 0x0F) {
                        // lost
                        i = 0; j = 28; k = 0;
                        g_state = G_STATE_LOSE;
                        drawSprite(g_mouseOver_col*8, g_mouseOver_row*8 + 28, 6);
                    }
                    else {
                        drawSprite(g_mouseOver_col*8, g_mouseOver_row*8 + 28, 7+g_clicked);
                        
                        if(g_clicked == 0x00) {
                            g_state = G_STATE_CASCADE;
                        }
                        
                    }
                    g_gameField[g_mouseOver] |= 0x10;
                    g_tilesRemaining--;
                }
                else {
                    // didn't left-click mouse
                    g_clicked = -1;
                    if((!g_rightButton) && g_rightButton_prev) {
                        if((g_gameField[g_mouseOver] & 0xF0) == 0x00) {
                            g_gameField[g_mouseOver] |= 0x20;
                            drawSprite(g_mouseOver_col*8, g_mouseOver_row*8 + 28, 2);
                            drawSprite(g_mouseOver_col*8, g_mouseOver_row*8 + 28, 4);
                        }
                        else if((g_gameField[g_mouseOver] & 0xF0) == 0x20) {
                            g_gameField[g_mouseOver] &= 0x0F;
                            drawSprite(g_mouseOver_col*8, g_mouseOver_row*8 + 28, 4);
                            drawSprite(g_mouseOver_col*8, g_mouseOver_row*8 + 28, 2);
                        }
                    }
                }
                
                // detect winning condition (only 10 unclicked squares)
                if((g_state != G_STATE_LOSE) && (g_tilesRemaining == 10)) {
                    // won
                    i = 0; j = 28; k = 0;
                    g_state = G_STATE_WIN;
                }
                
            }
            else if(g_state == G_STATE_CASCADE) {
                // prepare for a cascade
                for(i = 0; i < 81; i++) g_inCascade[i] = 0;
                g_state++;
            }
            else if(g_state == G_STATE_CASCADE + 1) {
                // run recursive cascadeHelper to determine which squares are in cascade
                cascadeHelper(g_mouseOver_row, g_mouseOver_col, 1);
                g_inCascade[g_mouseOver] = 0;
                i = 0; j = 28; k = 0;
                g_state++;
            }
            else if(g_state < G_STATE_CASCADE + 16) {
                // redraw up to 6 tiles that were in the cascade
                for(k = 0; ((k < 6) && (i < 81)); k++) {
                    if(g_inCascade[i]) {
                        int8_t row = i / 9;
                        int8_t col = i % 9;
                        drawSprite(col*8, row*8 + 28, 2);
                        drawSprite(col*8, row*8 + 28, 7+g_gameField[i]);
                        g_gameField[i] |= 0x10;
                        g_tilesRemaining--;
                    }
                    i++;
                }
                g_state++;
            }
            else if(g_state == G_STATE_CASCADE + 16) {
                // end cascade
                g_state = 0;
            }
            else if(g_state < G_STATE_LOSE + 9) {
                // lose - draw one row of uncovered mine tiles
                for(k = 0; k < 72; k+=8) {
                    if(g_gameField[i] == 0x0F) {
                        drawSprite(k, j, 2);
                        drawSprite(k, j, 5);
                    }
                    else if(g_gameField[i] == 0x2F) {
                        drawSprite(k, j, 4);
                        drawSprite(k, j, 5);
                    }
                    i++;
                }
                j+=8;
                g_state++;
            }
            else if(g_state == G_STATE_LOSE + 9) {
                // draw losing message
                video_line(10, 20, 58, 20, 0);
                video_line(10, 21, 58, 21, 0);
                video_line(10, 22, 58, 22, 0);
                video_line(10, 23, 58, 23, 0);
                video_line(10, 24, 58, 24, 0);
                video_putsmalls(18, 20, "YOU LOSE");
                g_state = G_STATE_END;
            }
            else if(g_state < G_STATE_WIN + 9) {
                // win - draw one row of flagged mine tiles
                for(k = 0; k < 72; k+=8) {
                    if(g_gameField[i] == 0x0F) {
                        drawSprite(k, j, 2);
                        drawSprite(k, j, 4);
                    }
                    i++;
                }
                j+=8;
                g_state++;
            }
            else if(g_state == G_STATE_WIN + 9) {
                // draw winning message
                video_line(10, 20, 58, 20, 0);
                video_line(10, 21, 58, 21, 0);
                video_line(10, 22, 58, 22, 0);
                video_line(10, 23, 58, 23, 0);
                video_line(10, 24, 58, 24, 0);
                video_putsmalls(14, 20, "GREAT JOB");
                video_line(50, 20, 50, 22, 1);
                video_line(52, 20, 52, 22, 1);
                video_pt(50, 24, 1);
                video_pt(52, 24, 1);
                g_state = G_STATE_END;
            }
            
            // draw cursor
            drawCursor(g_xCursor, g_yCursor);
        } // line 231
        
        // Read from RX MCU
        PINB_read   = PINB;
        PINB_read_0 = PINB;
        PINB_read_1 = PINB;
        PINB_read_2 = PINB;
        if(!((PINB_read == PINB_read_0)
          && (PINB_read == PINB_read_1)
          && (PINB_read == PINB_read_2))) {
            PINB_read = 0;
        }
        PORTD = (PORTD & 0b00011111) | (PINB_read & 0b11100000);
        decodePINB(PINB_read);
        if(!(PINC & 0b00000001)) {
            // mode of operation has switched to USB
            break;
        }
    } // while
} // main_game

void main_USB(void) {
    char PINB_read = 0;
    char PINB_read_0 = 0;
    char PINB_read_1 = 0;
    char PINB_read_2 = 0;
    uint8_t click_delay = 0;
    
    while(1) {
        wdt_reset();
        usbPoll();
        if(usbInterruptIsReady()){
            // Called after every poll of the interrupt endpoint
            // Happens about every 32ms (31.25Hz)
            // Prepare information to be sent to computer at next interrupt
            reportBuffer.buttonMask = rx_usb_buf.buttonMask;
            reportBuffer.dx     = (rx_usb_buf.dxH << 4) | rx_usb_buf.dxL;
            reportBuffer.dy     = (rx_usb_buf.dyH << 4) | rx_usb_buf.dyL;
            reportBuffer.dWheel = (rx_usb_buf.dWheelH << 4) | rx_usb_buf.dWheelL;
            if(rx_usb_buf.buttonMask) {
                if(!click_delay) {
                    reportBuffer.dx = (reportBuffer.dx);
                    reportBuffer.dy = (reportBuffer.dy);
                    click_delay = 2;
                }
                else {
                    click_delay--;
                    reportBuffer.dx = 0;
                    reportBuffer.dy = 0;
                }
            }
            usbSetInterrupt((void *)&reportBuffer, sizeof(reportBuffer));
        }
        
        // Read from RX MCU
        PINB_read   = PINB;
        PINB_read_0 = PINB;
        PINB_read_1 = PINB;
        PINB_read_2 = PINB;
        if(!((PINB_read == PINB_read_0)
          && (PINB_read == PINB_read_1)
          && (PINB_read == PINB_read_2))) {
            PINB_read = 0;
        }
        PORTD = (PORTD & 0b00011111) | (PINB_read & 0b11100000);
        decodePINB(PINB_read);
        if(PINC & 0b00000001) {
            // mode of operation has switched to game
            break;
        }
    }
}

int main(void) {
    init();
    while(1) {
        rand();
        reinit();
        if(operationMode)
            main_game();
        else
            main_USB();
    }
}

/****************************************************************
 * Initialization Routines                                      *
 ****************************************************************/

void init_game(void) {
    uint16_t i;
    
    // Disable watchdog
    wdt_disable();
    WDTCSR = 0; // clear watchdog timer control status register
    MCUSR |= (1 << WDRF); // clear watchdog reset flag
    // Enable Timer1 COMPA interrupt
    TIMSK1 = (1 << OCIE0A);
    
    // Initialize synch constants 
    LineCount = 1;
    syncON  = 0b00000000;
    syncOFF = 0b01000000;
    
    // Init game variables
    g_xCursor            = 0;
    g_yCursor            = 28;
    g_leftButton         = 0;
    g_leftButton_prev    = 0;
    g_middleButton       = 0;
    g_rightButton        = 0;
    g_rightButton_prev   = 0;
    g_vx                 = 0;
    g_vy                 = 0;
    g_mouseOver          = 0;
    g_mouseOver_row      = 0;
    g_mouseOver_col      = 0;
    g_mouseOver_prev     = 0;
    g_mouseOver_row_prev = 0;
    g_mouseOver_col_prev = 0;
    g_counting           = 0;
    setupField();
    g_minesRemaining     = 10;
    g_tilesRemaining     = 81;
    g_clicked            = -1;
    g_state              = 0;
    g_timeCntr           = 60;
    g_timeSec            = 0;
    
    // Init screen
    for(i = 0; i < 900; i++) screen[i] = 0;
    
    // Draw debug info at top
    video_line( 0,  0, 71,  0, 1);
    video_line(71,  0, 71, 27, 1);
    video_line(71, 27,  0, 27, 1);
    video_line( 0, 27,  0,  0, 1);
    video_line( 0, 10, 71, 10, 1);
    video_line( 0, 18, 71, 18, 1);
    video_line( 0, 26, 71, 26, 1);
    video_line(34, 10, 34, 18, 1);
    video_puts( 2,  2, "M");
    video_puts( 7,  2, "INESWEEPER");
    video_line(67,  2, 67,  6, 1);
    video_line(69,  2, 69,  6, 1);
    video_pt(67, 8, 1);
    video_pt(69, 8, 1);
    video_putsmalls( 4, 12, "ECE4760");
    video_putsmalls(37, 12, "TIME 000");
    video_putsmalls(10, 20, "SS868   JFE5");
    video_line(36, 20, 34, 24, 1);
    
    // Draw initial game field
    for(i = 0; i < 81; i++) {
        uint8_t row = i / 9;
        uint8_t col = i % 9;
        drawSprite((col*8), (row*8 + 28), 2);
    }
    
    // Draw initial cursor sprite
    drawCursor(g_xCursor, g_yCursor);
}

void reinit(void) {
    uint8_t i;
    
    cli(); // do not allow interrupts during reinitialization
    TIMSK1 &= 0b11111101; // Disable Timer1 COMPA interrupt
    
    usbDeviceDisconnect(); // enforce re-enumeration, do this while interrupts are disabled!
    i = 0;
    while(--i){            // fake USB disconnect for > 250 ms
        wdt_reset();
        _delay_ms(1);
    }
    
    // Select mode of operation - game or USB mouse
    operationMode = PINC & 0b00000001;
    if(operationMode) {
        init_game();
    }
    else {
        usbDeviceConnect();
    }
    
    sei();
    set_sleep_mode(SLEEP_MODE_IDLE);
    sleep_enable();
}

void init(void) {

    DDRB  = 0b00000000; // PORT B is an input
    DDRC  = 0b01100000; // PORT C controls video out but receives mode toggle
    DDRD  = 0b11100000; // PORT D outputs ACK. USB needs inputs without pullup.
    PORTC = 0;
    
    // Init Timer1 to generate sync
    OCR1A  = lineTime;               // One NTSC line
    TCCR1B = (1 << WGM12) | (0b001); // Clear-on-match | full speed
    TCCR1A = 0;                      // Turn off pwm and oc lines
    
    // Empty out received info from RX.
    rx_usb_buf.buttonMask = 0;
    rx_usb_buf.dxL        = 0;
    rx_usb_buf.dxH        = 0;
    rx_usb_buf.dyL        = 0;
    rx_usb_buf.dyH        = 0;
    rx_usb_buf.dWheelL    = 0;
    rx_usb_buf.dWheelH    = 0;
    
    // USB init stuff
    // Even if you don't use the watchdog, turn it off here. On newer devices,
    // the status of the watchdog (on/off, period) is PRESERVED OVER RESET!
    wdt_enable(WDTO_1S);
    // RESET status: all port bits are inputs without pull-up.
    // That's the way we need D+ and D-. Therefore we don't need any
    // additional hardware initialization.
    usbInit();
    
    // Seed RNG
    srand(0);
}

